function _toPrimitive(t, r) {
  if ("object" != typeof t || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r || "default");
    if ("object" != typeof i) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return ("string" === r ? String : Number)(t);
}
function _toPropertyKey(t) {
  var i = _toPrimitive(t, "string");
  return "symbol" == typeof i ? i : i + "";
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}

class BaseEventBus {
  constructor() {
    _defineProperty(this, "_eventDestructors", new Set());
  }
  get eventDestructors() {
    return this._eventDestructors;
  }
  addListener(event, listener) {
    event.on(listener);
    const unsub = () => event.off(listener);
    this._eventDestructors.add(unsub);
    return unsub;
  }
}

class TypedEvent {
  constructor() {
    _defineProperty(this, "listeners", []);
    _defineProperty(this, "listenersOncer", []);
    _defineProperty(this, "on", listener => {
      this.listeners.push(listener);
      return {
        dispose: () => this.off(listener)
      };
    });
    _defineProperty(this, "once", listener => {
      this.listenersOncer.push(listener);
      return {
        dispose: () => this.offOnce(listener)
      };
    });
    _defineProperty(this, "off", listener => {
      const callbackIndex = this.listeners.indexOf(listener);
      if (callbackIndex > -1) this.listeners.splice(callbackIndex, 1);
    });
    _defineProperty(this, "offOnce", listener => {
      const callbackIndex = this.listenersOncer.indexOf(listener);
      if (callbackIndex > -1) this.listenersOncer.splice(callbackIndex, 1);
    });
    _defineProperty(this, "clear", () => {
      this.listeners.length = 0;
      this.listenersOncer.length = 0;
    });
    _defineProperty(this, "emit", event => {
      this.listeners.forEach(listener => listener(event));
      this.listenersOncer.forEach(listener => listener(event));
      this.listenersOncer = [];
    });
    _defineProperty(this, "pipe", te => this.on(e => te.emit(e)));
  }
}

/* eslint-disable no-restricted-syntax */
/* eslint-disable no-await-in-loop */

class PriorityEvent {
  constructor() {
    _defineProperty(this, "listeners", []);
  }
  on(listener, priority = 0) {
    listener.priority = priority;
    this.listeners.push(listener);
    return () => this.off(listener);
  }
  off(listener) {
    const index = this.listeners.indexOf(listener);
    this.listeners.splice(index, 1);
  }
  clear() {
    this.listeners.length = 0;
  }
  sort(a, b) {
    if (a.priority === undefined) {
      a.priority = 0;
    }
    if (b.priority === undefined) {
      b.priority = 0;
    }
    if (a.priority > b.priority) {
      return -1;
    }
    if (a.priority < b.priority) {
      return 1;
    }
    return 0;
  }
  async emit(event) {
    for (const l of this.listeners.sort(this.sort)) {
      try {
        await l(event);
      } catch (e) {
        if (e) {
          if (e.message === 'reject') {
            throw e;
          }
        }
      }
    }
  }
}

export { BaseEventBus, PriorityEvent, TypedEvent };
