"use strict";
/* eslint-disable no-restricted-syntax */
/* eslint-disable no-await-in-loop */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriorityEvent = void 0;
class PriorityEvent {
    listeners = [];
    on(listener, priority = 0) {
        listener.priority = priority;
        this.listeners.push(listener);
        return () => this.off(listener);
    }
    off(listener) {
        const index = this.listeners.indexOf(listener);
        this.listeners.splice(index, 1);
    }
    clear() {
        this.listeners.length = 0;
    }
    sort(a, b) {
        if (a.priority === undefined) {
            a.priority = 0;
        }
        if (b.priority === undefined) {
            b.priority = 0;
        }
        if (a.priority > b.priority) {
            return -1;
        }
        if (a.priority < b.priority) {
            return 1;
        }
        return 0;
    }
    async emit(event) {
        for (const l of this.listeners.sort(this.sort)) {
            try {
                await l(event);
            }
            catch (e) {
                if (e) {
                    if (e.message === 'reject') {
                        throw e;
                    }
                }
            }
        }
    }
}
exports.PriorityEvent = PriorityEvent;
//# sourceMappingURL=priority-event.js.map