"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypedEvent = void 0;
class TypedEvent {
    listeners = [];
    listenersOncer = [];
    on = (listener) => {
        this.listeners.push(listener);
        return {
            dispose: () => this.off(listener),
        };
    };
    once = (listener) => {
        this.listenersOncer.push(listener);
        return {
            dispose: () => this.offOnce(listener),
        };
    };
    off = (listener) => {
        const callbackIndex = this.listeners.indexOf(listener);
        if (callbackIndex > -1)
            this.listeners.splice(callbackIndex, 1);
    };
    offOnce = (listener) => {
        const callbackIndex = this.listenersOncer.indexOf(listener);
        if (callbackIndex > -1)
            this.listenersOncer.splice(callbackIndex, 1);
    };
    clear = () => {
        this.listeners.length = 0;
        this.listenersOncer.length = 0;
    };
    emit = (event) => {
        this.listeners.forEach((listener) => listener(event));
        this.listenersOncer.forEach((listener) => listener(event));
        this.listenersOncer = [];
    };
    pipe = (te) => this.on((e) => te.emit(e));
}
exports.TypedEvent = TypedEvent;
//# sourceMappingURL=typedevent.js.map