import babel from '@rollup/plugin-babel';
import { nodeResolve } from '@rollup/plugin-node-resolve';
import nodePolyfills from 'rollup-plugin-node-polyfills';
import json from '@rollup/plugin-json';

const extensions = ['.ts', '.js'];

const babelConfig = babel({
  extensions,
  presets: [
    [
      '@babel/preset-env',
      {
        targets: {
          chrome: '61',
        },
      },
    ],
    ['@babel/preset-typescript'],
  ],
  plugins: [
    '@babel/plugin-transform-typescript',
    '@babel/plugin-syntax-dynamic-import',
    '@babel/plugin-proposal-class-properties',
  ],
});

const nodeConfig = nodeResolve({ extensions: ['.ts', '.js', '.json'] });

export default [
  {
    input: 'src/index.ts',
    output: [
      {
        file: 'dist/index.umd.js',
        format: 'umd',
        name: 'TypedEvent',
      },
      {
        file: 'dist/index.esm.js',
        format: 'esm',
      },
    ],
    plugins: [nodeConfig, babelConfig, nodePolyfills(), json()],
  },
];
