import { TypedEvent, Listener } from './typedevent';

export interface IBaseEventBus {
  eventDestructors: WeakSet<(...args: any[]) => any>;
  destroy(): void;
}

export abstract class BaseEventBus implements IBaseEventBus {
  protected _eventDestructors: WeakSet<(...args: any[]) => any> = new Set();

  public get eventDestructors() {
    return this._eventDestructors;
  }

  public abstract destroy(): void;

  public addListener<T>(event: TypedEvent<T>, listener: Listener<T>) {
    event.on(listener);
    const unsub = () => event.off(listener);
    this._eventDestructors.add(unsub);
    return unsub;
  }
}
