/* eslint-disable no-restricted-syntax */
/* eslint-disable no-await-in-loop */

interface Listener<T> {
  (event: T): any;
  priority?: number;
}

export class PriorityEvent<T> {
  private listeners: Listener<T>[] = [];

  on(listener: Listener<T>, priority = 0) {
    listener.priority = priority;
    this.listeners.push(listener);
    return () => this.off(listener);
  }

  off(listener: Listener<T>) {
    const index = this.listeners.indexOf(listener);
    this.listeners.splice(index, 1);
  }

  clear() {
    this.listeners.length = 0;
  }

  sort(a: Listener<T>, b: Listener<T>) {
    if (a.priority === undefined) {
      a.priority = 0;
    }
    if (b.priority === undefined) {
      b.priority = 0;
    }
    if (a.priority > b.priority) {
      return -1;
    }
    if (a.priority < b.priority) {
      return 1;
    }
    return 0;
  }

  async emit(event: T) {
    for (const l of this.listeners.sort(this.sort)) {
      try {
        await l(event);
      } catch (e: any) {
        if (e) {
          if (e.message === 'reject') {
            throw e;
          }
        }
      }
    }
  }
}
